local addon, private = ...
local core = LibStub("AceAddon-3.0"):GetAddon("addon")
local L = LibStub("AceLocale-3.0"):GetLocale("addon")
local ACR = LibStub("AceConfigRegistry-3.0")
local version = GetAddOnMetadata(addon, "X-Version")
local author = GetAddOnMetadata(addon, "X-Author")
local builddate = GetAddOnMetadata(addon, "X-Build")
local playerName = UnitName("player")
local pairs, type = pairs, type
local getOpt, setOpt

do
    function getOpt(info)
        local key = info[#info]
        return core.db.profile[key]
    end

    function setOpt(info, value)
        local key = info[#info]
        core.db.profile[key] = value
        core:applySettings()
    end
end

local BCMDropDownMenu = CreateFrame("Frame", "BCMDropDownMenu")
BCMDropDownMenu.displayMode = "MENU"
BCMDropDownMenu.info = {}
BCMDropDownMenu.uncheck = function(dropdownbutton)
    _G[dropdownbutton:GetName().."Check"]:Hide()
end

local info = {}
BCMDropDownMenu.initialize = function(self, level)
    if not level then return end
    local info = self.info
    wipe(info)
    if level == 1 then
        info.keepShownOnClick = 1
        info.isTitle = 1
        info.text = "Broker CrystalMote"
        info.notCheckable = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = 1
        info.text = L["Tracking Options"]
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.disabled = nil
        info.hasArrow = 1
        info.text = ("|cff00ff00"..L["The Burning Crusade"].."|r")
        info.value = "submenu1"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff0066ff"..L["Wrath of the Lich King"].."|r")
        info.value = "submenu2"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cffff6600"..L["Cataclysm"].."|r")
        info.value = "submenu3"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff19ac87"..L["Mists of Pandaria"].."|r")
        info.value = "submenu4"
        info.func = self.uncheck
        UIDropDownMenu_AddButton(info, level)
        info.hasArrow = nil
        info.notCheckable = nil
        info.text = L["Show Icons"]
        info.func = function() db.showIcons = not db.showIcons core:applySettings() end
        info.checked = function() return db.showIcons end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Short Text"]
        info.func = function() db.showShort = not db.showShort core:applySettings() end
        info.checked = function() return db.showShort end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons Only"]
        info.func = function() db.iconsOnly = not db.iconsOnly core:applySettings() end
        info.checked = function() return db.iconsOnly end
        UIDropDownMenu_AddButton(info, level)
        info.notCheckable = 1
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = 1
        info.text = L["Tooltip Options"]
        UIDropDownMenu_AddButton(info, level)
        info.isTitle = nil
        info.notCheckable = nil
        info.disabled = nil
        info.text = ("|cff00ff00"..L["The Burning Crusade"].."|r")
        info.func = function() db.modules.cmbc = not db.modules.cmbc core:applySettings() end
        info.checked = function() return db.modules.cmbc end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff0066ff"..L["Wrath of the Lich King"].."|r")
        info.func = function() db.modules.cmlk = not db.modules.cmlk core:applySettings() end
        info.checked = function() return db.modules.cmlk end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cffff6600"..L["Cataclysm"].."|r")
        info.func = function() db.modules.cmcata = not db.modules.cmcata core:applySettings() end
        info.checked = function() return db.modules.cmcata end
        UIDropDownMenu_AddButton(info, level)
        info.text = ("|cff19ac87"..L["Mists of Pandaria"].."|r")
        info.func = function() db.modules.cmmop = not db.modules.cmmop core:applySettings() end
        info.checked = function() return db.modules.cmmop end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Zero Counts"]
        info.func = function() db.showZero = not db.showZero core:applySettings() end
        info.checked = function() return db.showZero end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Bank Counts"]
        info.func = function() db.showBank = not db.showBank core:applySettings() end
        info.checked = function() return db.showBank end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Alt Counts"]
        info.func = function() db.showAlts = not db.showAlts core:applySettings() end
        info.checked = function() return db.showAlts end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Include Guild Counts"]
        info.func = function() db.showGuild = not db.showGuild core:applySettings() end
        info.checked = function() return db.showGuild end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons"]
        info.func = function() db.tooltipIcons = not db.tooltipIcons core:applySettings() end
        info.checked = function() return db.tooltipIcons end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Short Text"]
        info.func = function() db.tooltipShowShort = not db.tooltipShowShort core:applySettings() end
        info.checked = function() return db.tooltipShowShort end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Show Icons Only"]
        info.func = function() db.tooltipIconsOnly = not db.tooltipIconsOnly core:applySettings() end
        info.checked = function() return db.tooltipIconsOnly end
        UIDropDownMenu_AddButton(info, level)
        info.text = L["Color lines in the tooltip."]
        info.func = function() db.lineColor = not db.lineColor core:applySettings() end
        info.checked = function() return db.lineColor end
        UIDropDownMenu_AddButton(info, level)
        info.notCheckable = 1
        info.text = ""
        info.disabled = 1
        UIDropDownMenu_AddButton(info, level)
        info.disabled = nil
        info.text = L["Close"]
        info.func = function() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)
    elseif level == 2 then
        if UIDROPDOWNMENU_MENU_VALUE == "submenu1" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.bcitems.id) do
                local color = private.bcitems.color[k]
                local name = private.bcitems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text    = item
                info.func    = function() chardb.tracking[fixeditem] = not chardb.tracking[fixeditem] core:applySettings() end
                info.checked = function() return chardb.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu2" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.lkitems.id) do
                local color = private.lkitems.color[k]
                local name = private.lkitems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text    = item
                info.func    = function() chardb.tracking[fixeditem] = not chardb.tracking[fixeditem] core:applySettings() end
                info.checked = function() return chardb.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu3" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.cataitems.id) do
                local color = private.cataitems.color[k]
                local name = private.cataitems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text    = item
                info.func    = function() chardb.tracking[fixeditem] = not chardb.tracking[fixeditem] core:applySettings() end
                info.checked = function() return chardb.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
        if UIDROPDOWNMENU_MENU_VALUE == "submenu4" then
            info.keepShownOnClick = 1
            for k,itemid in pairs (private.mopitems.id) do
                local color = private.mopitems.color[k]
                local name = private.mopitems.strings[k]
                local fixeditem = strsub(itemid, 1, -1)
                local item  = "|cff"..color..name.."|r"
                info.text    = item
                info.func    = function() chardb.tracking[fixeditem] = not chardb.tracking[fixeditem] core:applySettings() end
                info.checked = function() return chardb.tracking[fixeditem] end
                UIDropDownMenu_AddButton(info, level)
            end
        end
    end
end

local options, moduleOptions = nil, {}
local function getOptions()
    if not options then
         options = {
            type = "group",
            name = "Broker CrystalMote",
            args = {
                display = {
                    order = 5,
                    type = "group",
                    inline = true,
                    name = "",
                    args = {
                        author = {
                            order = 10,
                            type = "description",
                            name = L["Author: "] .. author .. "    "..L["Version: "] .. version .. "    "..L["Build Date: "] .. builddate
                        },
                        counts = {
                            type = "group",
                            name = L["Display Options"],
                            guiInline = true,
                            order = 15,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                showZero = {
                                    type = "toggle",
                                    order = 20,
                                    width = "double",
                                    name = L["Include Zero Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showBank = {
                                    type = "toggle",
                                    order = 25,
                                    width = "double",
                                    name = L["Include Bank Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showAlts = {
                                    type = "toggle",
                                    order = 30,
                                    width = "double",
                                    name = L["Include Alt Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showGuild = {
                                    type = "toggle",
                                    order = 35,
                                    width = "double",
                                    name = L["Include Guild Counts"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                            },
                        },
                        tooltip = {
                            type = "group",
                            name = L["Tooltip Options"],
                            guiInline = true,
                            order = 40,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                scale = {
                                    order = 45,
                                    type = "range",
                                    name = L["Tooltip Scale"],
                                    min = .5,
                                    max = 1.5,
                                    step = .05,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                scroll= {
                                    order = 50,
                                    type = "range",
                                    name = L["Tooltip Scroll Height"],
                                    min = 100,
                                    max = 900,
                                    step = 25,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                updatetime = {
                                    order = 55,
                                    type = "range",
                                    name = L["Update Time"],
                                    min = 1,
                                    max = 30,
                                    step = 1,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                scantime = {
                                    order = 60,
                                    type = "range",
                                    name = L["Guild Bank Scan Time"],
                                    min = 5,
                                    max = 30,
                                    step = 1,
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipIcons = {
                                    type = "toggle",
                                    order = 65,
                                    width = "double",
                                    name = L["Show icons in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipShowShort = {
                                    type = "toggle",
                                    order = 70,
                                    width = "double",
                                    name = L["Show Short Text"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                tooltipIconsOnly = {
                                    type = "toggle",
                                    order = 75,
                                    width = "double",
                                    name = L["Show icons only in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                    hidden = function() return not db.tooltipIcons end
                                },
                                lineColor = {
                                    type = "toggle",
                                    order = 80,
                                    width = "double",
                                    name = L["Color lines in the tooltip."],
                                    get = getOpt,
                                    set = setOpt,
                                    hidden = function() return not db.tooltipIcons end
                                },
                            },
                        },
                        tracking = {
                            type = "group",
                            name = L["Tracking Options"],
                            guiInline = true,
                            order = 85,
                            get = getOpt,
                            set = setOpt,
                            args = {
                                showIcons= {
                                    type = "toggle",
                                    order = 90,
                                    width = "double",
                                    name = L["Show Icons"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                showShort = {
                                    type  = "toggle",
                                    order = 95,
                                    width = "double",
                                    name = L["Show Short Text"],
                                    get = getOpt,
                                    set = setOpt,
                                },
                                iconsOnly = {
                                    type = "toggle",
                                    order = 100,
                                    width = "double",
                                    name = L["Show Icons Only"],
                                    get = getOpt,
                                    set = setOpt,
                                    hidden = function() return not db.showIcons end
                                },
                            },
                        },
                        purge = {
                            type = "group",
                            name = L["Delete Character"],
                            guiInline = true,
                            order = 105,
                            args = {
                                deleteChar= {
                                    type = "select",
                                    order = 110,
                                    name = L["Delete Character"],
                                    desc = L["This action will force a UI reload."],
                                    values = private.charsList,
                                    set = function(v, alt)
                                            core:purgeData(alt)
                                    end
                                },
                            },
                        },
                    },
                },
            },
        }
        for k,v in pairs(moduleOptions) do
            options.args[k] = (type(v) == "function") and v() or v
        end
    end
    return options
end

function core:SetupOptions()
    self.optFrames = {}
    ACR:RegisterOptionsTable(addon, getOptions)
    self.optFrames.core = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addon, "Broker CrystalMote", nil, "display")
    self.optFrames.core.default = function() core:SetDefaultOptions() end
    self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles")
end

function core:SetDefaultOptions()
    self.db:ResetProfile()
    ACR:NotifyChange(addon)
end

function core:RegisterModuleOptions(name, optTable, displayName)
    moduleOptions[name] = optTable
    self.optFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addon, displayName or name, "Broker CrystalMote", name)
end

function core:purgeData(alt)
   charsdb[alt] = nil
   ReloadUI()
end